#ifndef TRXRUN_H
#define TRXRUN_H
/** \file
   Definitions for running the MC kernel.
*/
/*
   
   Copyright  1992-2011 M.Kraemer, GSI Darmstadt 

   $Log: trxrun.h,v $
   Revision 1.3  2009/05/29 12:43:21  kraemer
   reworked projectiles, targets

   Revision 1.2  2002/11/25 16:35:23  kraemer
   external fields, new cutoff handling, volume mode

   Revision 1.1  2001/01/09 14:54:39  kraemer
   *** empty log message ***


*/

#include "cmderr.h"
#include "trxproj.h"
#include "trxtarg.h"
#include "trxrnd.h"
#include "trxxs.h"
#include "trxgeo.h"
#include "trxsrc.h"
#include "trxparticle.h"
#include "trxlmd.h"

                                      /* */
                                      /* masks */
#define TRXRUN_MASKGLOBAL 0xFF000000  /* global (e.g. debug) */
#define TRXRUN_MASKFUNC   0x00FF0000  /* function specific flags */
#define TRXRUN_MASKQUAL   0x0000FFFF  /* for additional qualifiers */

                                      /* flags */
#define TRXRUN_DEBUG      0x80000000  /* general check flag */
#define TRXRUN_CHECK      TRXRUN_DEBUG   /* general check flag */
#define TRXRUN_QUIET      0x10000000  /* suppress error messages */
#define TRXRUN_NOMSG      TRXRUN_QUIET


#undef TRXRUN_201007xx

                                      /** \struct STRXRUN
                                          \brief run kernel structures.
                                      */ 
struct STRXRUN {
   unsigned int uiFlag;               /**< flag bits */
   unsigned int uiEvt, uiEvtMax;
   uint32_t ui32CPU, ui32Elapsed;     /**< timing (msec) */
                                      /**< interaction filter mask */
   unsigned int uiFilterMaskIA[TRXPROJ_MAX];
   int mP, nP;                        /**< max/current # particles */
   struct STRXPARTICLE *psP;          /**< [mP] particle stack */ 
};

#define MTRXRUN_STACKINC 128          /* particle stack increment */
/*

   Functions to run the MC simulation
   ==================================
   
   TRXRunDiscard       ( psRun, ui32MaskPT, ui32MaskIA, uiFlag );
                       o discard specified particles / interactions from run stack
                     6 x to test
		                     o calls:

   TRXRunFree          ( run, flag )
                       o release run resources
                     3 x to test
                       o calls:
		                       o TRXParticleFree()
			                      o TRXRunInit()

   TRXRunImportFilter  ( run, str, flag )
                       o store output filter options
                     4 x to test
                       o calls:

   TRXRunInit          ( run, flag )
                       o initialize a run handle
                     2 x to test
                       o calls:

   TRXRunInspect       ( run, flag )
                       o inspect a run handle
                     1 x to test
                       o calls:
		                       o TRXParticleInspect()

   TRXRunRun           ( run, src, geo, xsbase, rnd, lmd, flag )
                       o run the simulation   
                     7 o to test
                       ! remove absorbed (photo etc) primaries
                       o LmdWrite() when no file defined
		                     o calls:
		                       o TRXParticleInit()            
		                       o TRXParticleEvt()            
                         o TRXLmdWrite()
                         o ...
			 
   TRXRunStackInc      ( run, inc, flag ) 
                       o increment particle stack
                     5 x to test
                       x use realloc()
		                     o calls:
		   

   TRXRunError   : error handling

*/

                                      /* function prototypes */
struct SCMDERR TRXRunDiscard( struct STRXRUN *, uint32_t, uint32_t, unsigned int ); 
struct SCMDERR TRXRunFree   ( struct STRXRUN *, unsigned int ); 
struct SCMDERR TRXRunImportFilter( struct STRXRUN *, const char *, unsigned int ); 
struct SCMDERR TRXRunInit   ( struct STRXRUN *, unsigned int ); 
struct SCMDERR TRXRunInspect( struct STRXRUN *, unsigned int ); 

struct SCMDERR TRXRunRun    ( struct STRXRUN *, 
                   struct STRXSRC *,
	                  struct STRXGEO *,
	                  struct STRXXSBASE *, 
                   struct STRXRND *,
                   struct STRXLMD *,
		                 unsigned int );
struct SCMDERR TRXRunStackInc( struct STRXRUN *, unsigned int, unsigned int ); 

struct STRXRUN _TRXRunTest( unsigned int );
                                      /* error handling */
int TRXRunError( struct SCMDERR *, unsigned int );

enum { ERRTRXRUN_SYS=1,
       ERRTRXRUN_BADADDR,
       ERRTRXRUN_SAMPLETOTALFAILED, 
       ERRTRXRUN_BADFILTER,
       ERRTRXRUN_SAMPLEIACTFAILED, 
       ERRTRXRUN_SAMPLESRCFAILED,
       ERRTRXRUN_LOCATEFAILED,
       ERRTRXRUN_ISECTFAILED,  
       ERRTRXRUN_BADPROJ,
};

#endif
